@@ -1,19 +1,20 @@
 """Licensed under GPLv3, see https://www.gnu.org/licenses/"""
+# pylint: disable=too-many-lines
 
 import os
 import shutil
 from glob import glob
 from multiprocessing.pool import ThreadPool
 from pathlib import Path
-from typing import TYPE_CHECKING
+from typing import TYPE_CHECKING, ClassVar
 
 from .args import parse_args
 from .aur import find_aur_packages, get_repo_url
 from .config import (
-    AUR_REPOS_CACHE_PATH,
-    BUILD_CACHE_PATH,
-    BUILD_DEPS_LOCK,
-    PACKAGE_CACHE_PATH,
+    AurReposCachePath,
+    BuildCachePath,
+    BuildDepsLockPath,
+    PackageCachePath,
     PikaurConfig,
 )
 from .core import (
@@ -21,16 +22,21 @@ from .core import (
     DataType,
     dirname,
     interactive_spawn,
-    isolate_root_cmd,
     joined_spawn,
+    mkdir,
     open_file,
     remove_dir,
     replace_file,
-    running_as_root,
     spawn,
-    sudo,
 )
-from .exceptions import BuildError, CloneError, DependencyError, DependencyNotBuiltYetError, SysExit
+from .exceptions import (
+    BuildError,
+    CloneError,
+    DependencyError,
+    DependencyNotBuiltYetError,
+    SkipBuildError,
+    SysExit,
+)
 from .filelock import FileLock
 from .i18n import translate, translate_many
 from .logging import create_logger
@@ -38,6 +44,7 @@ from .makepkg_config import MakePkgCommand, MakepkgConfig, get_pkgdest
 from .pacman import PackageDB, get_pacman_command, install_built_deps
 from .pprint import (
     ColorsHighlight,
+    TTYRestoreContext,
     bold_line,
     color_enabled,
     color_line,
@@ -45,6 +52,11 @@ from .pprint import (
     print_stderr,
     print_stdout,
 )
+from .privilege import (
+    isolate_root_cmd,
+    sudo,
+    using_dynamic_users,
+)
 from .prompt import (
     ask_to_continue,
     get_editor_or_exit,
@@ -53,7 +65,7 @@ from .prompt import (
 )
 from .srcinfo import SrcInfo
 from .updates import is_devel_pkg
-from .urllib import wrap_proxy_env
+from .urllib_helper import wrap_proxy_env
 from .version import VersionMatcher, compare_versions
 
 if TYPE_CHECKING:
@@ -78,7 +90,7 @@ def _shell(cmds: list[str]) -> "InteractiveSpawn":
     return interactive_spawn(isolate_root_cmd(wrap_proxy_env(cmds)))
 
 
-def _mkdir(to_path: Path) -> None:
+def isolated_mkdir(to_path: Path) -> None:
     mkdir_result = spawn(isolate_root_cmd(["mkdir", "-p", str(to_path)]))
     if mkdir_result.returncode != 0:
         print_stdout(mkdir_result.stdout_text)
@@ -90,10 +102,10 @@ def copy_aur_repo(from_path: Path, to_path: Path) -> None:
     from_path = from_path.resolve()
     to_path = to_path.resolve()
     if not to_path.exists():
-        _mkdir(to_path)
+        isolated_mkdir(to_path)
 
     from_paths = []
-    for src_path_str in glob(f"{from_path}/*") + glob(f"{from_path}/.*"):
+    for src_path_str in glob(f"{from_path}/*") + glob(f"{from_path}/.*"):  # noqa: PTH207
         src_path = Path(src_path_str)
         if src_path.name not in IGNORE_PATHS_WHEN_COPYING:
             from_paths.append(src_path)
@@ -105,7 +117,7 @@ def copy_aur_repo(from_path: Path, to_path: Path) -> None:
     if result.returncode != 0:
         if to_path.exists():
             remove_dir(to_path)
-            _mkdir(to_path)
+            isolated_mkdir(to_path)
         result = interactive_spawn(cmd_args)
         if result.returncode != 0:
             raise RuntimeError(translate(f"Can't copy '{from_path}' to '{to_path}'."))
@@ -169,11 +181,11 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
                 self.package_base = pkgbase
             else:
                 no_pkgname_error = translate("Can't get package name from PKGBUILD")
-                raise BuildError(no_pkgname_error)
+                raise BuildError(message=no_pkgname_error, build=self)
         elif package_names:
             self.package_names = package_names
             self.package_base = find_aur_packages([package_names[0]])[0][0].packagebase
-            self.repo_path = AUR_REPOS_CACHE_PATH / self.package_base
+            self.repo_path = AurReposCachePath()() / self.package_base
             self.pkgbuild_path = self.repo_path / DEFAULT_PKGBUILD_BASENAME
         else:
             missing_property_error = translate(
@@ -184,7 +196,7 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
             )
             raise NotImplementedError(missing_property_error)
 
-        self.build_dir = BUILD_CACHE_PATH / self.package_base
+        self.build_dir = BuildCachePath()() / self.package_base
         logger.debug("Build dir: {}", self.build_dir)
         self.build_gpgdir = self.args.build_gpgdir
         self.built_packages_paths = {}
@@ -199,7 +211,7 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
             else:
                 self.clone = True
         else:
-            self.repo_path.mkdir(parents=True)
+            isolated_mkdir(self.repo_path)
             self.clone = True
 
         self.reviewed = self.current_hash == self.last_installed_hash
@@ -208,6 +220,13 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
         self._local_pkgs_with_build_deps = set()
         self._local_provided_pkgs_with_build_deps = {}
 
+    def git_diff(self) -> "InteractiveSpawn":
+        return _shell([
+            "git",
+            "-C", str(self.repo_path),
+            "diff",
+        ])
+
     def git_reset_changed(self) -> "InteractiveSpawn":
         return _shell([
             "git",
@@ -249,7 +268,7 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
                 get_repo_url(self.package_base),
                 str(self.repo_path),
             ]
-        result: "InteractiveSpawn | None" = None
+        result: InteractiveSpawn | None = None
         if cmd_args:
             result = spawn(isolate_root_cmd(wrap_proxy_env(cmd_args)))
         self.reviewed = self.current_hash == self.last_installed_hash
@@ -286,7 +305,9 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
         with open_file(git_hash_path) as current_hash_file:
             return current_hash_file.readlines()[0].strip()
 
-    def get_latest_dev_sources(self, *, check_dev_pkgs: bool = True) -> None:
+    def get_latest_dev_sources(
+            self, *, check_dev_pkgs: bool = True, tty_restore: bool = False,
+    ) -> None:
         if not self.reviewed:
             return
         self.prepare_build_destination()
@@ -305,7 +326,7 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
             ).format(
                 bold_line(", ".join(self.package_names)),
             ),
-        ))
+        ), tty_restore=tty_restore)
         pkgver_result = joined_spawn(
             isolate_root_cmd(
                 [*MakePkgCommand.get(), "--nobuild", "--nocheck", "--nodeps"],
@@ -315,16 +336,67 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
         )
         if pkgver_result.returncode != 0:
             error_text = translate("failed to retrieve latest dev sources:")
-            print_stderr()
-            print_stderr()
-            print_error(error_text)
-            print_stderr(pkgver_result.stdout_text)
-            if (
-                    not self.args.skip_failed_build
-            ) and (
-                not ask_to_continue(default_yes=False)
-            ):
-                raise BuildError(error_text)
+            print_stderr(tty_restore=tty_restore)
+            print_stderr(tty_restore=tty_restore)
+            print_error(error_text, tty_restore=tty_restore)
+            print_stderr(pkgver_result.stdout_text, tty_restore=tty_restore)
+
+            if self.args.skip_failed_build:
+                answer = translate("s")
+            elif self.args.noconfirm:
+                answer = translate("a")
+            else:  # pragma: no cover
+                prompt = "{} {}\n{}\n> ".format(
+                    color_line("::", ColorsHighlight.yellow),
+                    translate("Try recovering?"),
+                    "\n".join((
+                        translate("[R] retry clone"),
+                        translate("[d] delete build dir and try again"),
+                        translate("[e] edit PKGBUILD"),
+                        translate("[i] ignore the error"),
+                        "-" * 24,
+                        translate("[s] skip building this package"),
+                        translate("[a] abort building all the packages"),
+                    )),
+                )
+                answer = get_input(
+                    prompt,
+                    translate("r").upper() +
+                    translate("d") +
+                    translate("e") +
+                    translate("i") +
+                    translate("s") +
+                    translate("a"),
+                )
+
+            answer = answer.lower()[0]
+            if answer == translate("r"):  # pragma: no cover
+                self.get_latest_dev_sources(check_dev_pkgs=check_dev_pkgs)
+                return
+            if answer == translate("d"):  # pragma: no cover
+                self.prepare_build_destination(flush=True)
+                self.get_latest_dev_sources(check_dev_pkgs=check_dev_pkgs)
+                return
+            if answer == translate("e"):  # pragma: no cover
+                editor_cmd = get_editor_or_exit()
+                if editor_cmd:
+                    interactive_spawn(
+                        [*editor_cmd, str(self.pkgbuild_path)],
+                    )
+                    interactive_spawn(isolate_root_cmd([
+                        "cp",
+                        str(self.pkgbuild_path),
+                        str(self.build_dir / DEFAULT_PKGBUILD_BASENAME),
+                    ]))
+                    raise PkgbuildChanged
+                self.get_latest_dev_sources(check_dev_pkgs=check_dev_pkgs)
+                return
+            if answer == translate("i"):
+                return
+            if answer == translate("a"):
+                raise SysExit(125)
+            # "s"kip
+            raise SkipBuildError(message=error_text, build=self)
         SrcInfo(self.build_dir).regenerate()
         self._source_repo_updated = True
 
@@ -472,7 +544,7 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
                 pkg_path = (Path(pkg_dest) if pkg_dest else self.build_dir) / pkg_path
                 logger.debug("Resolving full path: {} from base path: {}", pkg_path, pkg_basename)
             new_package_path = (
-                Path(pkg_dest) if pkg_dest else PACKAGE_CACHE_PATH
+                Path(pkg_dest) if pkg_dest else PackageCachePath()()
             ) / pkg_basename
             logger.debug("New package path: {}", new_package_path)
             if not pkg_dest or MakePkgCommand.pkgdest_skipped:
@@ -480,8 +552,7 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
                 new_package_sig_path = new_package_path.parent / (
                     new_package_path.name + ".sig"
                 )
-                if not PACKAGE_CACHE_PATH.exists():
-                    PACKAGE_CACHE_PATH.mkdir(parents=True)
+                mkdir(PackageCachePath()())
                 replace_file(pkg_path, new_package_path)
                 replace_file(pkg_sig_path, new_package_sig_path)
             pkg_path = new_package_path
@@ -594,7 +665,7 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
         self._local_provided_pkgs_with_build_deps = PackageDB.get_local_provided_dict()
 
     def install_all_deps(self, all_package_builds: dict[str, "PackageBuild"]) -> None:
-        with FileLock(BUILD_DEPS_LOCK):
+        with FileLock(BuildDepsLockPath()()):
             self.get_deps(all_package_builds)
             if self.all_deps_to_install or self.built_deps_to_install:
                 PackageDB.discard_local_cache()
@@ -691,9 +762,52 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
             ) and (
                 not ask_to_continue(default_yes=False)
             ):
-                raise BuildError(error_text)
+                raise BuildError(message=error_text, build=self)
             self.skip_carch_check = True
 
+    def _run_makepkg_cmd(
+            self,
+            makepkg_args: list[str],
+            *,
+            skip_pgp_check: bool,
+            skip_file_checksums: bool,
+            skip_check: bool,
+            no_prepare: bool,
+    ) -> "InteractiveSpawn":
+        cmd_args = MakePkgCommand.get() + makepkg_args
+        if skip_pgp_check:
+            cmd_args += ["--skippgpcheck"]
+        if skip_file_checksums:
+            cmd_args += ["--skipchecksums"]
+        if self.skip_carch_check:
+            cmd_args += ["--ignorearch"]
+        if skip_check:
+            cmd_args += ["--nocheck"]
+        if no_prepare:
+            cmd_args += ["--noprepare"]
+
+        env = {}
+        if self.build_gpgdir:
+            env["GNUPGHOME"] = self.build_gpgdir
+
+        cmd_args = isolate_root_cmd(cmd_args, cwd=self.build_dir, env=env)
+        spawn_kwargs: SpawnArgs = {
+            "cwd": str(self.build_dir),
+            "env": {**os.environ, **env},
+        }
+        if self.args.hide_build_log:
+            spawn_kwargs.update({
+                "stdout": PIPE,
+                "stderr": PIPE,
+            })
+
+        result = interactive_spawn(
+            cmd_args,
+            **spawn_kwargs,
+        )
+        print_stdout()
+        return result
+
     def build_with_makepkg(self) -> bool:  # pylint: disable=too-many-branches,too-many-statements
         makepkg_args = []
         if not self.args.needed:
@@ -711,38 +825,13 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
         skip_check = False
         no_prepare = False
         while True:
-            cmd_args = MakePkgCommand.get() + makepkg_args
-            if skip_pgp_check:
-                cmd_args += ["--skippgpcheck"]
-            if skip_file_checksums:
-                cmd_args += ["--skipchecksums"]
-            if self.skip_carch_check:
-                cmd_args += ["--ignorearch"]
-            if skip_check:
-                cmd_args += ["--nocheck"]
-            if no_prepare:
-                cmd_args += ["--noprepare"]
-
-            env = {}
-            if self.build_gpgdir:
-                env["GNUPGHOME"] = self.build_gpgdir
-
-            cmd_args = isolate_root_cmd(cmd_args, cwd=self.build_dir, env=env)
-            spawn_kwargs: "SpawnArgs" = {
-                "cwd": str(self.build_dir),
-                "env": {**os.environ, **env},
-            }
-            if self.args.hide_build_log:
-                spawn_kwargs.update({
-                    "stdout": PIPE,
-                    "stderr": PIPE,
-                })
-
-            result = interactive_spawn(
-                cmd_args,
-                **spawn_kwargs,
+            result = self._run_makepkg_cmd(
+                makepkg_args=makepkg_args,
+                skip_pgp_check=skip_pgp_check,
+                skip_file_checksums=skip_file_checksums,
+                skip_check=skip_check,
+                no_prepare=no_prepare,
             )
-            print_stdout()
             build_succeeded = result.returncode == 0
             if build_succeeded:
                 break
@@ -750,7 +839,9 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
             print_stderr(
                 color_line(
                     translate("Command '{}' failed to execute.").format(
-                        " ".join(cmd_args),
+                        " ".join(str(a) for a in result.args)
+                        if isinstance(result.args, list)
+                        else result.args,
                     ),
                     ColorsHighlight.red,
                 ),
@@ -849,13 +940,13 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
 
         if not build_succeeded:
             self.failed = True
-            raise BuildError
+            raise BuildError(message="failed to build", build=self)
         self._set_built_package_path()
 
 
 class AlreadyClonedRepos:
 
-    repos: list[str] = []
+    repos: ClassVar[list[str]] = []
 
     @classmethod
     def add(cls, repo: str) -> None:
@@ -876,13 +967,17 @@ def clone_aur_repos(package_names: list[str]) -> dict[str, PackageBuild]:
         for pkgbase, pkg_names in packages_bases.items()
         if not AlreadyClonedRepos.get(pkgbase)
     }
+
     pool_size: int | None = None
     if clone_c := parse_args().aur_clone_concurrency:
         pool_size = clone_c
-    elif running_as_root():
+    elif using_dynamic_users():
         pool_size = 1
     exc: CloneError | None = None
-    with ThreadPool(processes=pool_size) as pool:
+    with (
+            TTYRestoreContext(),
+            ThreadPool(processes=pool_size) as pool,
+    ):
         requests = {
             key: pool.apply_async(repo_status.update_aur_repo, ())
             for key, repo_status in package_builds_by_base.items()
@@ -900,6 +995,7 @@ def clone_aur_repos(package_names: list[str]) -> dict[str, PackageBuild]:
                 AlreadyClonedRepos.add(package_base)
     if exc:
         raise exc
+
     all_package_builds_by_base = {
         pkgbase: PackageBuild(pkg_names)
         for pkgbase, pkg_names in packages_bases.items()
