@@ -15,12 +15,10 @@ from .config import DiffPagerValues, PikaurConfig
 from .conflicts import find_aur_conflicts
 from .core import (
     PackageSource,
+    chown_to_current,
     interactive_spawn,
-    isolate_root_cmd,
     open_file,
     remove_dir,
-    running_as_root,
-    sudo,
 )
 from .exceptions import (
     BuildError,
@@ -29,6 +27,7 @@ from .exceptions import (
     DependencyNotBuiltYetError,
     DependencyVersionMismatchError,
     PackagesNotFoundInAURError,
+    SkipBuildError,
     SysExit,
 )
 from .i18n import translate
@@ -42,9 +41,9 @@ from .pacman import (
     refresh_pkg_db_if_needed,
     strip_repo_name,
 )
-from .pikspect import TTYRestore
 from .pprint import (
     ColorsHighlight,
+    TTYRestoreContext,
     bold_line,
     color_line,
     print_error,
@@ -59,6 +58,11 @@ from .print_department import (
     print_package_downgrading,
     print_package_uptodate,
 )
+from .privilege import (
+    isolate_root_cmd,
+    sudo,
+    using_dynamic_users,
+)
 from .prompt import (
     ask_to_continue,
     get_editor_or_exit,
@@ -97,17 +101,15 @@ def edit_file(filename: str | Path) -> bool:  # pragma: no cover
     if not editor_cmd:
         return False
     old_hash = hash_file(filename)
-    sub_tty = TTYRestore()
-    TTYRestore.restore()
-    interactive_spawn([
-        *editor_cmd, str(filename),
-    ])
-    sub_tty.restore_new()
+    with TTYRestoreContext(before=True, after=True):
+        interactive_spawn([
+            *editor_cmd, str(filename),
+        ])
     new_hash = hash_file(filename)
     return old_hash != new_hash
 
 
-class InstallPackagesCLI():
+class InstallPackagesCLI:
 
     # User input
     args: "PikaurArgs"
@@ -156,9 +158,31 @@ class InstallPackagesCLI():
         self.transactions = {}
         self.failed_to_build_package_names = []
 
+        try:
+            self._handle_refresh()
+        except self.ExitMainSequence:
+            return
+
+        if self.args.sysupgrade and not self.args.repo:
+            print_stderr("{} {}".format(  # pylint: disable=consider-using-f-string
+                color_line("::", ColorsHighlight.blue),
+                bold_line(translate("Starting full AUR upgrade...")),
+            ))
+        if self.args.aur:
+            self.not_found_repo_pkgs_names = self.install_package_names
+            self.install_package_names = []
+        if self.args.pkgbuild:
+            self.get_info_from_pkgbuilds()
+
+        self.main_sequence()
+
+    def _handle_refresh(self) -> None:
         if not self.args.aur and (self.args.sysupgrade or self.args.refresh):
 
-            with ThreadPool() as pool:
+            with (
+                    ThreadPool() as pool,
+                    TTYRestoreContext(),
+            ):
                 threads = []
                 if self.args.sysupgrade:
                     self.news = News()
@@ -175,25 +199,12 @@ class InstallPackagesCLI():
                 pool.join()
 
             if not (self.install_package_names or self.args.sysupgrade):
-                return
+                raise self.ExitMainSequence
 
             if self.args.refresh:
                 PackageDB.discard_repo_cache()
                 print_stdout()
 
-        if self.args.sysupgrade and not self.args.repo:
-            print_stderr("{} {}".format(  # pylint: disable=consider-using-f-string
-                color_line("::", ColorsHighlight.blue),
-                bold_line(translate("Starting full AUR upgrade...")),
-            ))
-        if self.args.aur:
-            self.not_found_repo_pkgs_names = self.install_package_names
-            self.install_package_names = []
-        if self.args.pkgbuild:
-            self.get_info_from_pkgbuilds()
-
-        self.main_sequence()
-
     class ExitMainSequence(Exception):  # noqa: N818
         """Raise when need to finish Install CLI"""
 
@@ -411,6 +422,7 @@ class InstallPackagesCLI():
         with NamedTemporaryFile() as tmp_file:
             with open_file(tmp_file.name, "w") as write_file:
                 write_file.write(text_before)
+            chown_to_current(Path(tmp_file.name))
             edit_file(tmp_file.name)
             with open_file(tmp_file.name, "r") as read_file:
                 selected_packages = parse_pkg_names(read_file.read())
@@ -580,6 +592,7 @@ class InstallPackagesCLI():
                         "\n".join((
                             translate("Try recovering?"),
                             translate("[T] try again"),
+                            translate("[d] git diff"),
                             translate("[c] git checkout -- '*'"),
                             # translate("[c] git checkout -- '*' ; git clean -f -d -x"),
                             translate("[r] remove dir and clone again"),
@@ -590,11 +603,16 @@ class InstallPackagesCLI():
                     )
                     answer = get_input(
                         prompt,
-                        translate("t").upper() + translate("c") + translate("r")
+                        translate("t").upper() + translate("d")
+                        + translate("c") + translate("r")
                         + translate("s") + translate("a"),
                     )
 
                 answer = answer.lower()[0]
+                if answer == translate("d"):  # pragma: no cover
+                    print_stdout()
+                    package_build.git_diff()
+                    print_stdout()
                 if answer == translate("c"):  # pragma: no cover
                     package_build.git_reset_changed()
                 elif answer == translate("p"):  # pragma: no cover
@@ -715,28 +733,41 @@ class InstallPackagesCLI():
         full_filename = package_build.repo_path / filename
         return edit_file(full_filename)
 
-    def _get_installed_status(self) -> None:
+    def _get_installed_status(self) -> None:  # pylint: disable=too-many-branches
         all_package_builds = set(self.package_builds_by_name.values())
 
         # if running as root get sources for dev packages synchronously
         # (to prevent race condition in systemd dynamic users)
         num_threads: int | None = None
-        if running_as_root():  # pragma: no cover
+        if using_dynamic_users():  # pragma: no cover
             num_threads = 1
 
         # check if pkgs versions already installed
         # (use threads because devel packages require downloading
         # latest sources for quite a long time)
-        with ThreadPool(processes=num_threads) as pool:
+        with (
+                ThreadPool(processes=num_threads) as pool,
+                TTYRestoreContext(),
+        ):
             threads = []
             for pkg_build in all_package_builds:
+                def callback(pkg_build_inner: PackageBuild) -> None:
+                    pkg_build_inner.get_latest_dev_sources(
+                        check_dev_pkgs=self.args.needed,
+                        tty_restore=True,
+                    )
                 threads.append(
-                    pool.apply_async(getattr, (pkg_build, "version_already_installed")),
+                    pool.apply_async(callback, (pkg_build, )),
                 )
-            for thread in threads:
-                thread.get()
-            pool.close()
-            pool.join()
+            try:
+                for thread in threads:
+                    thread.get()
+                pool.close()
+                pool.join()
+            except SkipBuildError as exc:
+                all_package_builds.remove(exc.build)
+                for pkg_name in exc.build.package_names:
+                    self.discard_install_info(pkg_name)
 
         # handle if version is already installed
         if not self.args.needed:
@@ -750,7 +781,12 @@ class InstallPackagesCLI():
             for package_name in pkg_build.package_names:
                 if package_name not in local_db:
                     continue
-                if pkg_build.version_already_installed:
+                try:
+                    already_installed = pkg_build.version_already_installed
+                except SkipBuildError:
+                    self.discard_install_info(package_name)
+                    continue
+                if already_installed:
                     print_package_uptodate(package_name, PackageSource.AUR)
                     self.discard_install_info(package_name)
                 elif (
