@@ -374,7 +374,10 @@ Gonna fetch install info for:
             return []
         all_local_pkgs = PackageDB.get_local_dict()
         pkg_install_infos = []
-        for pkg in find_sysupgrade_packages(ignore_pkgs=self.manually_excluded_packages_names):
+        for pkg in find_sysupgrade_packages(
+                ignore_pkgs=self.manually_excluded_packages_names,
+                install_pkgs=self.install_package_names,
+        ):
             local_pkg = all_local_pkgs.get(pkg.name)
             install_info = RepoInstallInfo(
                 name=pkg.name,
@@ -525,9 +528,10 @@ Gonna fetch install info for:
                 self.package_is_ignored(pkg_name)
             ):
                 del aur_updates_install_info_by_name[pkg_name]
-            for pkg_list in self.pkgbuilds_packagelists.values():
-                if pkg_name in pkg_list:
-                    del aur_updates_install_info_by_name[pkg_name]
+            else:
+                for pkg_list in self.pkgbuilds_packagelists.values():
+                    if pkg_name in pkg_list:
+                        del aur_updates_install_info_by_name[pkg_name]
         self.aur_updates_install_info += list(aur_updates_install_info_by_name.values())
         logger.debug("got AUR pkgs install info: {}", self.aur_updates_install_info)
 
@@ -608,7 +612,7 @@ Gonna fetch install info for:
         """Update packages' install info to show deps in prompt."""
         all_provided_pkgs = PackageDB.get_repo_provided_dict()
         all_local_pkgnames = PackageDB.get_local_pkgnames()
-        all_deps_install_infos: "Sequence[InstallInfo]" = (
+        all_deps_install_infos: Sequence[InstallInfo] = (
             self.new_repo_deps_install_info +
             self.new_thirdparty_repo_deps_install_info +
             self.aur_deps_install_info  # type: ignore[operator]
@@ -638,10 +642,10 @@ Gonna fetch install info for:
             ):
                 providing_for = [
                     pkg_name for pkg_name in sum([
-                        [
+                        next(
                             [vm.line, vm.pkg_name]
                             for vm in (VersionMatcher(prov), )
-                        ][0]
+                        )
                         for prov in provides
                     ], [])
                     if pkg_name in all_requested_pkg_names
